{-# LINE 29 "ConnectedComponents.lhs" #-}
  module Algos.Graphs.ConnectedComponents ( CC
                                          , ccLabelling
                                          , ccPartition
                                          , componentLabelling
                                          , componentPartition
                                          ) where
 
  import Algos.Graphs.Forest
  import Algos.Graphs.Graph
  import Algos.Graphs.DFS (dfs)
  import Data.Array (array,(!),Ix)
{-# LINE 61 "ConnectedComponents.lhs" #-}
  newtype CC = CC Int deriving (Eq,Ord,Ix)
 
  ccLabelling  ::  AdjList v vl el -> V -> CC
  ccPartition  ::  AdjList v vl el -> [AdjList v vl el]
{-# LINE 71 "ConnectedComponents.lhs" #-}
  ccLabelling g = componentLabelling (gNumVertices (adjGraph g)) CC (dfs g)
 
  componentLabelling          ::  Int -> (Int -> a) -> Forest V E -> V -> a
  componentLabelling n l f v  =   array (1, n) cls ! (vIx v)
    where  cvs  =  map (map vIx) (componentVertices f)
           cls  =  concat $ zipWith zip cvs (map (repeat . l) [1..])
 
  componentVertices  ::  Forest V E -> [[V]]
  componentVertices  =   map preOrder . trees
{-# LINE 124 "ConnectedComponents.lhs" #-}
  ccPartition g = componentPartition g (dfs g)
{-# LINE 128 "ConnectedComponents.lhs" #-}
  componentPartition         ::  AdjList v vl el -> Forest V E -> [AdjList v vl el]
  componentPartition g f     =   adjs
 
    where  cvs  =      componentVertices f
           ces  =      map (filter sameComponent . concatMap vOutEdges) cvs
 
           sameComponent e = cIx (eTail e) == cIx (eHead e)
 
           n = gNumVertices (adjGraph g)
 
           vIxs'    =     concat $ zipWith (\vs i -> zipWith (\v j -> (vIx v, (j, i))) vs [1..]) cvs [1..]
           vIxA'    =     array (1,n) vIxs'
           vIx'  v  =     fst   (vIxA' ! vIx v)
           cIx   v  =     snd   (vIxA' ! vIx v)
 
           cvsi   =     map (map mkVertex) cvs
           cesi   =     map (map mkEdge) ces
 
           gs = zipWith Graph cvsi cesi
 
           adjsi   =     map makeDirAdjListIx gs
           adjs    =     map dropVertexIndices adjsi
 
           mkVertex  v     =  Vertex  (vIx' v)                           (adjVertexId g v,adjVertexLabel g v)
           mkEdge    e     =  Edge    (vIx' (eTail e)) (vIx' (eHead e))  (adjEdgeLabel g e)
 
           dropVertexIndices g' = g'   {  adjVertexId     =  makeVAttr ids
                                       ,  adjVertexLabel  =  makeVAttr labs
                                       }
             where    ids   =  map (\v -> (vIx v, fst  $ adjVertexLabel g' v)) (gVertices (adjGraph g'))
                      labs  =  map (\v -> (vIx v, snd  $ adjVertexLabel g' v)) (gVertices (adjGraph g'))
