{-# LINE 32 "Forest.lhs" #-}
  {-#  LANGUAGE MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances  #-}
 
  module Algos.Graphs.Forest ( Forest(..)
                             , Tree(..)
                             , Subtree(..)
                             , UpTree(..)
                             , preOrder
                             , postOrder
                             , levelOrder
                             , toUpTree
                             , toForestEq
                             , toForestOrd
                             , toForestIx
                             , toForestIndexByEq
                             , toForestIndexByOrd
                             , toForestIndexByIx
                             ) where
 
  import Algos.DS.Dict
  import Data.Ix (Ix)
  import Data.List (partition)
{-# LINE 80 "Forest.lhs" #-}
  newtype  Forest   v e  = Forest { trees :: [Tree v e] }
  data     Tree     v e  = Tree v [Subtree v e]
  data     Subtree  v e  = Subtree e (Tree v e)
{-# LINE 90 "Forest.lhs" #-}
  preOrderForest  ::  Forest v e -> [v]
  preOrderForest  =   concatMap preOrderTree . trees
 
  preOrderTree     ::  Tree v e -> [v]
  preOrderTree t   =   go t []
    where go (Tree v ws) vs = v : foldr go vs (map (\(Subtree _ t') -> t') ws)
{-# LINE 101 "Forest.lhs" #-}
  postOrderForest  ::  Forest v e -> [v]
  postOrderForest  =   concatMap postOrderTree . trees
 
  postOrderTree     ::  Tree v e -> [v]
  postOrderTree t   =   go t []
    where go (Tree v ws) vs = foldr go (v:vs) (map (\(Subtree _ t') -> t') ws)
{-# LINE 111 "Forest.lhs" #-}
  levelOrderForest  ::  Forest v e -> [v]
  levelOrderForest  =   concatMap levelOrderTree . trees
 
  levelOrderTree    ::  Tree v e -> [v]
  levelOrderTree t  =   levelOrder' [t]
 
  levelOrder'     ::  [Tree v e] -> [v]
  levelOrder' []  =   []
  levelOrder' ts  =   map (\(Tree v _) -> v) ts ++ 
                      levelOrder' (concatMap (\(Tree _ ws) -> map (\(Subtree _ t) -> t) ws) ts)
{-# LINE 148 "Forest.lhs" #-}
  type UpTree v e = [(v, [(e, v)])]
{-# LINE 170 "Forest.lhs" #-}
  toForestEq  ::  Eq v => UpTree v e -> Forest v e
  toForestEq  =   toForestIndexByEq id
 
  toForestOrd   ::  Ord v => UpTree v e -> Forest v e
  toForestOrd   =   toForestIndexByOrd id
 
  toForestIx   ::  Ix v => UpTree v e -> Forest v e
  toForestIx   =   toForestIndexByIx id
 
  toForestIndexByEq    ::  Eq k => (v -> k) -> UpTree v e -> Forest v e
  toForestIndexByEq f  =   toForest f makeEqDict eqDictFind
 
  toForestIndexByOrd     ::  Ord k => (v -> k) -> UpTree v e -> Forest v e
  toForestIndexByOrd f   =   toForest f makeOrdDict ordDictFind
 
  toForestIndexByIx     ::  Ix k => (v -> k) -> UpTree v e -> Forest v e
  toForestIndexByIx f   =   toForest f makeIxDict ixDictFind
{-# LINE 190 "Forest.lhs" #-}
  toForest                         ::  (v -> k) -> ([k] -> [(k,Subtree v e)] -> d) -> (d -> k -> [Subtree v e]) ->
                                       UpTree v e -> Forest v e
  toForest f makeDict findDict t   =   Forest $ map (\(v,_) -> Tree v (findDict d (f v))) roots
    where  (roots,nonroots)  =  partition (null . snd) t
           d                 =  makeDict  (map (f . fst) t)
                                          (map  (\(v,(e,p):_) -> (f p,Subtree e $ Tree v $ findDict d (f v)))
                                                nonroots)
{-# LINE 217 "Forest.lhs" #-}
  toUpTreeT    ::  Tree v e -> UpTree v e
  toUpTreeT t  =   tut t []
    where tut (Tree v ws) as = (v,as) : concatMap (\(Subtree e t) -> tut t ((e,v) : as)) ws
 
  toUpTreeF  ::  Forest v e -> UpTree v e
  toUpTreeF  =   concatMap toUpTree . trees
{-# LINE 227 "Forest.lhs" #-}
  class TreeLike t v e | t -> v, t -> e where
    preOrder     ::  t -> [v]
    postOrder    ::  t -> [v]
    levelOrder   ::  t -> [v]
    toUpTree     ::  t -> UpTree v e
 
  instance TreeLike (Forest v e) v e where
    preOrder    =  preOrderForest
    postOrder   =  postOrderForest
    levelOrder  =  levelOrderForest
    toUpTree    =  toUpTreeF
{-# LINE 241 "Forest.lhs" #-}
  instance TreeLike (Tree v e) v e where
    preOrder    =  preOrderTree
    postOrder   =  postOrderTree
    levelOrder  =  levelOrderTree
    toUpTree    =  toUpTreeT
